//+FHDR--------------------------------------------------------------------------------------------------------
//  Company: 
//-----------------------------------------------------------------------------------------------------------------
//  File Name             :    IIR_Filter.v
//  Department            :    
//  Author                :    thfu
//  Author's Tel          :     
//-----------------------------------------------------------------------------------------------------------------
//  Relese History
//  Version     Date            Author          Description
//  0.4         2024-05-28      thfu
//2024-05-28 10:22:49 
//-----------------------------------------------------------------------------------------------------------------
//  Keywords            :       
//
//-----------------------------------------------------------------------------------------------------------------
//  Parameter
//
//-----------------------------------------------------------------------------------------------------------------
//  Purpose                 :
//                      
//-----------------------------------------------------------------------------------------------------------------
//  Target Device:        
//  Tool versions:        
//-----------------------------------------------------------------------------------------------------------------
//  Reuse Issues
//  Reset Strategy: 
//  Clock Domains: 
//  Critical Timing:
//  Asynchronous I/F:
//  Synthesizable (y/n): 
//  Other:
//-FHDR--------------------------------------------------------------------------------------------------------
module  IIR_Filter (
                         rstn,
                         en,
                         clk,
                         din_re,
                         din_im,
                         a_re,
                         a_im,
                         b_re,
                         b_im,
                         dout
                   );

input	                 rstn;
input	                 clk;
input	                 en;
input   signed	[15:0]   din_re;
input   signed	[15:0]   din_im;
input   signed	[36:0]   a_re;
input   signed	[36:0]   a_im;
input   signed	[20:0]   b_re;
input   signed	[20:0]   b_im;

output  signed	[15:0]   dout;

wire	signed	[31:0] 	 x1_re;
wire	signed	[31:0]	 x1_im;
wire	signed	[31:0] 	 x2_re;
wire	signed	[31:0]	 x2_im;
wire	signed	[31:0] 	 v_re;
wire	signed	[31:0]	 v_im;
reg 	signed	[31:0]	 v1_re;
reg 	signed	[31:0]	 v1_im;

wire    signed	[31:0]	 y_re;
wire    signed	[31:0]	 y_im;
wire    signed	[31:0]	 y1_re;
wire    signed	[31:0]	 y1_im;
wire    signed	[31:0]	 y2_re;
wire    signed	[31:0]	 y2_im;

reg 	signed	[15:0]   dout_re;

mult_C	#(16,16,37,37)	inst_c1         (
                                            .clk	(clk    	),
                                            .rstn	(rstn   	),
                                            .en		(en     	),
                                            .a		(din_re		),
                                            .b		(din_im		),
                                            .c		(a_re		),
                                            .d		(a_im		),
                                            .Re		(x1_re   	),
                                            .Im		(x1_im   	)
				);

mult_C	#(32,32,21,21)	inst_c2         (
                                            .clk	(clk    	),
                                            .rstn	(rstn   	),
                                            .en		(en     	),
                                            .a		(x1_re		),
                                            .b		(x1_im		),
                                            .c		(b_re	  	),
                                            .d		(b_im	  	),
                                            .Re		(x2_re     	),
                                            .Im		(x2_im     	)
				);

assign	v_re       =   x1_re - x2_re;
assign	v_im       =   x1_im - x2_im;

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
                v1_re    <=    'h0;
                v1_im    <=    'h0;
 		 end 
	 else if(en)
		 begin
                v1_re    <=    v_re;
                v1_im    <=    v_im;
		  end
	 else
		 begin
                v1_re    <=    v1_re;
                v1_im    <=    v1_im;
		  end

mult_C	#(32,32,21,21)	inst_c3         (
                                            .clk	(clk    	),
                                            .rstn	(rstn   	),
                                            .en		(en     	),
                                            .a		(y_re		),
                                            .b		(y_im		),
                                            .c		(b_re	  	),
                                            .d		(b_im	  	),
                                            .Re		(y1_re     	),
                                            .Im		(y1_im     	)
				);

mult_C	#(32,32,21,21)	inst_c4         (
                                            .clk	(clk    	),
                                            .rstn	(rstn   	),
                                            .en		(en     	),
                                            .a		(y1_re		),
                                            .b		(y1_im		),
                                            .c		(b_re	  	),
                                            .d		(b_im	  	),
                                            .Re		(y2_re     	),
                                            .Im		(y2_im     	)
				);

assign	y_re       =   v1_re + y2_re;
assign	y_im       =   v1_im + y2_im;

always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    dout_re  <= 'h0;
		 end 
	 else if(en)
		 begin
		    dout_re  <= y_re[31:16];
		  end
	 else
		 begin
		    dout_re  <= dout_re;
		  end
/*
always @(posedge clk or negedge rstn) 
  	if (!rstn)
		 begin
		    dout_r1 <= 'h0;
		 end 
	 else if(en)
		 begin
	     if(YsumR1_re[16:15]==2'b01)
			dout_r1	<=	16'd32767;
		 else if(YsumR1_re[16:15]==2'b10)
			dout_r1	<=	-16'd32768;
		 else
			dout_r1	<=	YsumR1_re[15:0];		 
		 end
	 else
		 begin
		    dout_r1  <= dout_r1;
		  end
*/
assign	dout = dout_re;

endmodule

